//*******************************************************
//
//	Z80EM1
//
//	Flash書き換え
//
//	2014.9.15	KAWAKAMI Yukio
//*******************************************************

#include	"flash.h"

/**
 * @brief IAP command code definitions
 */
typedef enum
{
    IAP_PREPARE = 50,       // Prepare sector(s) for write operation
    IAP_COPY_RAM2FLASH = 51,     // Copy RAM to Flash
    IAP_ERASE = 52,              // Erase sector(s)
    IAP_BLANK_CHECK = 53,        // Blank check sector(s)
    IAP_READ_PART_ID = 54,       // Read chip part ID
    IAP_READ_BOOT_VER = 55,      // Read chip boot code version
    IAP_COMPARE = 56,            // Compare memory areas
    IAP_REINVOKE_ISP = 57,       // Reinvoke ISP
    IAP_READ_SERIAL_NUMBER = 58, // Read serial number
	IAP_ERASE_PAGE = 59,		// Erase Page
}  IAP_COMMAND_CODE;

/**
 * @brief IAP status code definitions
 */
typedef enum
{
    CMD_SUCCESS,                 // 0 Command is executed successfully.
    INVALID_COMMAND,             // 1 Invalid command.
    SRC_ADDR_ERROR,              // 2 Source address is not on a word boundary.
    DST_ADDR_ERROR,              // 3 Destination address is not on a correct boundary.
    SRC_ADDR_NOT_MAPPED,         // 4 Source address is not mapped in the memory map.
    DST_ADDR_NOT_MAPPED,         // 5 Destination address is not mapped in the memory map.
    COUNT_ERROR,                 // 6 Byte count is not multiple of 4 or is not a permitted value.
    INVALID_SECTOR,              // 7 Sector number is invalid.
    SECTOR_NOT_BLANK,            // 8 Sector is not blank.
    SECTOR_NOT_PREPARED_FOR_WRITE_OPERATION,    // 9 Command to prepare sector for write operation was not executed.
    COMPARE_ERROR,               // 10 Source and destination data is not same.
    BUSY,                          // 11 Flash programming hardware interface is busy.
} IAP_STATUS_CODE;


/**
 * @brief IAP command structure
 */
typedef struct {
    uint32_t cmd;   // Command
    uint32_t param[4];      // Parameters
    uint32_t status;        // status code
    uint32_t result[4];     // Result
} IAP_COMMAND_Type;


typedef void (*IAP)(uint32_t *, uint32_t *);
static IAP IAP_Call = (IAP)0x1fff1ff1;

//*******************************************************
//	Flash Prepare処理
//*******************************************************
static int prepare(uint32_t adrs){
    IAP_COMMAND_Type command;

	command.cmd = IAP_PREPARE;					// Prepare Sector for Write
	command.param[0] = adrs / FLASH_SECTOR_SIZE;	// Start Sector
	command.param[1] = adrs / FLASH_SECTOR_SIZE;	// End Sector
	__disable_irq();
	IAP_Call(&command.cmd, &command.status);	// Call IAP Command
	__enable_irq();
	if (command.status != CMD_SUCCESS)	return (int)command.status;
	return 0;
}


//*******************************************************
//	Flash１ページ書き換え
//*******************************************************
int write_flash(const uint8_t *buff, const uint32_t adrs){
    IAP_COMMAND_Type command;
	int	res;

	// アドレス確認
	if (buff == NULL)		return 1;
	if (adrs >= 0x10000)	return 1;

	// Prepare
	res = prepare(adrs);
	if (res != CMD_SUCCESS)	return res;

	// Erase Page
	command.cmd = IAP_ERASE_PAGE;
	command.param[0] = adrs / FLASH_PAGE_SIZE;
	command.param[1] = adrs / FLASH_PAGE_SIZE;
	command.param[2] = SystemCoreClock / 1000;
	__disable_irq();
	IAP_Call(&command.cmd, &command.status);	// Call IAP Command
	__enable_irq();
	if (command.status != CMD_SUCCESS)	return (int)command.status;

	// Prepare
	res = prepare(adrs);
	if (res != CMD_SUCCESS)	return res;

	// Write Page
	command.cmd = IAP_COPY_RAM2FLASH;
	command.param[0] = adrs;
	command.param[1] = (uint32_t)buff;
	command.param[2] = FLASH_PAGE_SIZE;			// 256, 512, 1024, 4096
	command.param[3] = SystemCoreClock / 1000;
	__disable_irq();
	IAP_Call(&command.cmd, &command.status);	// Call IAP Command
	__enable_irq();
	if (command.status != CMD_SUCCESS)	return (int)command.status;

	return 0;
}


//*******************************************************
//	Flash１ページ Verify
//*******************************************************
int verify_flash(uint8_t *buff, uint32_t adrs){
	int	i;
	uint8_t *p = (uint8_t *)adrs;

	for (i = 0; i < FLASH_PAGE_SIZE; i++){
		if (*buff != *p)	return 1;
		buff++;
		p++;
	}

	return 0;
}

